class People:
    George_Washington = 'Q23'
    Descartes = 'Q9191'
    Leibniz = 'Q9047'
    John_von_Neumann = 'Q17455'


class Location:
    Beijing = 'Q956'
    China = 'Q29520'
    PRC = 'Q148'
    Asia = 'Q48'

    NewYorkCity = 'Q60'
    NewYorkState = 'Q1384'
    USA = 'Q30'


class WrittenWork:
    Capital = 'Q58784'
    The_Laws_of_Thought = 'Q7746455'
    Computing_Machinery_and_Intelligence = 'Q772056'
    A_Mathematical_Theory_of_Communication = 'Q724029'
    Theory_of_Games_and_Economic_Behavior = 'Q5226156'


class University:
    Gottingen = 'Q152838'
    Oxford = 'Q34217'
    MIT = 'Q49108'
    PKU = 'Q16952'


class Company:
    Apple = 'Q312'
    China_Resources = 'Q1073501'
    VW = 'Q246'
    Xerox = 'Q152433'


class CommonTypes:
    Human = 'Q5'

    LiteraryWork = 'Q7725634'
    WrittenWork = 'Q47461344'

    University = 'Q3918'
    Business = 'Q4830453'
